{
*****************************************************************************
*  Name    : HIDDescriptor                                                  *
*  Author  : David John Barker                                              *
*  Notice  : Copyright (c) 2007 Mecanique                                   *
*          : All Rights Reserved                                            *
*  Date    : 10/01/2007                                                     *
*  Version : 1.0                                                            *
*  Notes   : EasyHID USB HID descriptor module                              *
*****************************************************************************
}
module HIDDescriptor

// EasyHID options...
#option USB_HAVE_SERIAL_STRING = false
#option USB_VID                = 6017
#option USB_PID                = 2000
#option HID_EP_OUT_POLLING_MS  = $0A
#option HID_EP_IN_POLLING_MS   = $0A
#option USB_BUS_POWER          = $32
#option HID_REPORT_BYTES_IN    = $20
#option HID_REPORT_BYTES_OUT   = $20
#option USB_SERVICE            = true

// import modules...
include "usbconfig.bas"
include "usbdefs.bas"
         
// Device Descriptor 
public const device_dsc(18) as byte = (
	sizeof(USB_DEV_DSC),          // bLength - size of this descriptor IN bytes
	DSC_DEV,	                     // bDescriptorType - device descriptor type
	$00,	                        // bcdUSB (low byte)  - USB Spec Release Number IN BCD format (2.0)
	$02,	                        // bcdUSB (high byte) - USB Spec Release Number IN BCD format (2.0)
	$00,	                        // bDeviceClass - class Code
	$00,	                        // bDeviceSubClass - subclass code
	$00,	                        // bDeviceProtocol - protocol code
	USB_EP0_BUFF_SIZE,	         // bMaxPacketSize - max packet size for EP0
	USB_VID and $00FF,            // idVendor (low byte)
	USB_VID >> 8,                 // idVendor (high byte)
	USB_PID and $00FF,            // idProduct (low byte)
	USB_PID >> 8,                 // idProduct (high byte)
	USB_DEVICE_VERSION and $00FF, // bcdDevice (low byte) - device release number IN BCD format
	USB_DEVICE_VERSION >> 8,      // bcdDevice (high byte) - device release number IN BCD format
	USB_MANUFACTURER_INDEX,       // iManufacturer - manufacturer string index
	USB_PRODUCT_INDEX,            // iProduct - product string index
	USB_SERIAL_INDEX,             // iSerialNumber - device serial number string index
	$01	                        // bNumConfigurations - mumber of possible configurations
)
 
// Configuration 1 Descriptor
public const cfg01(41) as byte = (
    
   // configuration Descriptor (cd01)
   sizeof(USB_CFG_DSC),    // Size of this descriptor in bytes
   DSC_CFG,                // CONFIGURATION descriptor type
   41,                     // sizeof(cfg01) - Total length of data for this cfg
   0,                      // HIGH BYTE
   1,                      // Number of interfaces in this cfg
   1,                      // Index value of this configuration
   0,                      // Configuration string index
   _DEFAULT or _RWU,       // Attributes, see usbdefs_std_dsc.h
   USB_BUS_POWER,          // Max power consumption (2X mA)

   // interface descriptor (i00a00)           
   sizeof(USB_INTF_DSC),   // Size of this descriptor in bytes
   DSC_INTF,               // INTERFACE descriptor type
   0,                      // Interface Number
   0,                      // Alternate Setting Number
   2,                      // Number of endpoints in this intf
   HID_INTF,               // Class code
   0,                      // Subclass code
   0,                      // Protocol code
   0,                      // Interface string index
    
   // HID class-specific descriptor (hid_i00a00)
   sizeof(USB_HID_DSC),    // Size of this descriptor in bytes
   DSC_HID,                // HID descriptor type
   $10,                    // HID Spec Release Number in BCD format (Low)
   $01,                    // HID Spec Release Number in BCD format (High)   
   $00,                    // Country Code (0x00 for Not supported)
   HID_NUM_OF_DSC,         // Number of class descriptors, see usbcfg.h
   DSC_RPT,                // Report descriptor type
   HID_RPT01_SIZE,         // sizeof(hid_rpt01) - Size of the report descriptor
   0,                      // HIGH BYTE
   
   // endpoint descriptor (IN) (ep01i_i00a00)
   sizeof(USB_EP_DSC),		 // Size of this descriptor in bytes 
 	DSC_EP,						 // Endpoint descriptor type
	_EP01_IN,					 // Endpoint 1 IN	
	_INT,						    // Interrupt transfers
	HID_INT_IN_EP_SIZE,		 // Maximum packet size
	$00,                     // high byte
	HID_EP_IN_POLLING_MS,    // Polling interval (milliseconds)
	
	// enpoint descriptor (OUT) (ep01o_i00a00)
	sizeof(USB_EP_DSC),      // Size of this descriptor in bytes 
	DSC_EP,						 // Endpoint descriptor type
	_EP01_OUT,					 // Endpoint 1 OUT	
	_INT,						    // Interrupt transfers
	HID_INT_OUT_EP_SIZE,	    // Maximum packet size
	$00,                     // high byte
	HID_EP_OUT_POLLING_MS    // Polling interval (milliseconds)
)
  
// language string - unicode format...  
public const sd000(4) as byte = (
   4,              // Size of this descriptor in bytes 
   DSC_STR,        // DescriptorType
   $09,            // language ID - low byte   
   $04             // language ID - high byte 
)   

// manufacturer string - unicode format...
public const sd001(20) as byte = (
   20,
   DSC_STR,
   "M", 0, "y", 0, "C", 0, "o", 0, "m", 0, "p", 0, "a", 0, "n", 0, "y", 0
)


// product string - unicode format...
public const sd002(20) as byte = (
   20,
   DSC_STR,
   "M", 0, "y", 0, "P", 0, "r", 0, "o", 0, "d", 0, "u", 0, "c", 0, "t", 0
)


// product serial number...
#if USB_HAVE_SERIAL_STRING
// no serial string
#endif

(*
   This example report descriptor for a "generic HID" defines one
   report of each type. Each report contains two bytes of data with 
   a vendor-defined Usage.

   HID_RPT01_SIZE is defined in usbcfg.h and must equal the number of bytes 
   in the report descriptor.
*)
public const hid_rpt01(HID_RPT01_SIZE) as byte = (                             
   $06, $A0, $FF,	              // Usage page (vendor defined)                                
  	$09, $01,	                 // Usage ID (vendor defined)
  	$A1, $01,	                 // Collection (application)

	// The Input report
   $09, $03,     	              // Usage ID - vendor defined
   $15, $00,     	              // Logical Minimum (0)
   $26, $FF, $00,               // Logical Maximum (255)
   $75, $08,     	              // Report Size (8 bits)
   $95, HID_REPORT_BYTES_IN,    // Report Count (2 fields)
   $81, $02,     	              // Input (Data, Variable, Absolute)  

   // The Output report
   $09, $04,     	              // Usage ID - vendor defined
   $15, $00,     	              // Logical Minimum (0)
   $26, $FF, $00,               // Logical Maximum (255)
   $75, $08,     	              // Report Size (8 bits)
   $95, HID_REPORT_BYTES_OUT,   // Report Count (2 fields)
   $91, $02,                    // Output (Data, Variable, Absolute)  

   // The Feature report
   $09, $05,     	              // Usage ID - vendor defined
   $15, $00,     	              // Logical Minimum (0)
   $26, $FF, $00,               // Logical Maximum (255)
   $75, $08,		              // Report Size (8 bits)
   $95, $02, 		              // Report Count (2 fields)
   $B1, $02,     	              // Feature (Data, Variable, Absolute)  
 	$C0	                       // end collection
)
 
